<?php

namespace Drupal\druhels;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;

class CommerceHelper {

  /**
   * Return user carts.
   *
   * @return OrderInterface[]
   */
  public static function getCarts(AccountInterface $account = NULL): array {
    /** @see \Drupal\commerce_cart\Plugin\Block\CartBlock::build() */
    $carts = \Drupal::service('commerce_cart.cart_provider')->getCarts($account); /** @var OrderInterface[] $carts */
    return array_filter($carts, function (OrderInterface $cart) {
      return $cart->hasItems() && $cart->cart->value;
    });
  }

  /**
   * Return carts total price.
   */
  public static function getCartsTotalPrice(AccountInterface $account = NULL): Price {
    $carts_total_price = new Price(0, self::getDefaultCurrencyCode());

    if ($carts = self::getCarts($account)) {
      foreach ($carts as $cart) {
        if ($cart_total_price = $cart->getTotalPrice()) {
          $carts_total_price = $carts_total_price->add($cart_total_price);
        }
      }
    }

    return $carts_total_price;
  }

  /**
   * Clear carts.
   */
  public static function clearCarts(AccountInterface $account = NULL): void {
    foreach (self::getCarts($account) as $cart) {
      $cart->delete();
    }
  }

  /**
   * Return current store default currency code.
   */
  public static function getDefaultCurrencyCode(): string {
    $current_store = \Drupal::service('commerce_store.current_store')->getStore();
    return $current_store->getDefaultCurrencyCode();
  }

  /**
   * Return order total quantity.
   */
  public static function getOrderTotalQuantity(OrderInterface $order): int {
    $quantity = 0;

    foreach ($order->getItems() as $order_item) {
      $quantity += (int)$order_item->getQuantity();
    }

    return $quantity;
  }

  /**
   * Return carts items total quantity.
   */
  public static function getCartsTotalQuantity(): int {
    $quantity = 0;

    foreach (self::getCarts() as $cart) {
      $quantity += self::getOrderTotalQuantity($cart);
    }

    return $quantity;
  }

  /**
   * Return formatted price.
   *
   * Example:
   * <code>
   * $formatted_price = CommerceHelper::formatPrice(new Price(123, 'USD'));
   * // '$123'
   * </code>
   *
   * @param Price|array $price Price object or array
   */
  public static function formatPrice($price): string {
    if (!$price) {
      return '';
    }
    if (is_string($price)) {
      return $price;
    }

    if ($price instanceof Price) {
      $price = $price->toArray();
    }
    if (is_array($price) && isset($price['currency_code'], $price['number'])) {
      return \Drupal::service('commerce_price.currency_formatter')->format($price['number'], $price['currency_code']);
    }

    return (string)$price;
  }

  /**
   * Return TRUE if current page is product page.
   */
  public static function isProductPage(string $product_type = NULL, RouteMatchInterface $route_match = NULL, bool $clear_cache = FALSE): bool {
    static $current_product_type;

    if ($clear_cache) {
      $current_product_type = NULL;
    }

    if ($current_product_type === NULL) {
      if (!$route_match) {
        $route_match = \Drupal::routeMatch();
      }

      if ($route_match->getRouteName() == 'entity.commerce_product.canonical') {
        $current_product_type = $route_match->getParameter('commerce_product')->bundle();
      }
      else {
        $current_product_type = '';
      }
    }

    return $product_type ? ($current_product_type == $product_type) : (bool)$current_product_type;
  }

  /**
   * Return current product id.
   */
  public static function getCurrentProductId(RouteMatchInterface $route_match = NULL): ?int {
    if (!$route_match) {
      $route_match = \Drupal::routeMatch();
    }
    return $route_match->getRawParameter('commerce_product');
  }

  /**
   * Return current product.
   */
  public static function getCurrentProduct(RouteMatchInterface $route_match = NULL): ?ProductInterface {
    if ($product_id = self::getCurrentProductId($route_match)) {
      return Product::load($product_id);
    }
    return NULL;
  }

  /**
   * Return product render array.
   */
  public static function productView(ProductInterface $product, string $view_mode = 'full'): array {
    return \Drupal::entityTypeManager()->getViewBuilder('commerce_product')->view($product, $view_mode);
  }

}
