<?php

namespace Drupal\druhels;

use DateTimeInterface as PhpDateTimeInterface;
use Drupal\Core\Datetime\DrupalDateTime;

class DateHelper {

  /**
   * Return php DateTime object.
   *
   * Example:
   * <code>
   * $formatted_date = DateHelper::createPhpDateTime('2010-01-01')->format('d.m.Y');
   * // '01.01.2010'
   * </code>
   *
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date
   */
  public static function createPhpDateTime($date): ?PhpDateTimeInterface {
    $datetime = NULL;

    if (is_int($date)) {
      $datetime = new \DateTime("@$date");
    }
    elseif (is_string($date)) {
      $datetime = new \DateTime($date);
    }
    elseif ($date instanceof DrupalDateTime) {
      $datetime = $date->getPhpDateTime();
    }
    elseif ($date instanceof PhpDateTimeInterface) {
      $datetime = $date;
    }

    return $datetime;
  }

  /**
   * Return days count between two dates.
   *
   * Example:
   * <code>
   * $days = DateHelper::getDaysBetweenDates('2010-01-01', '2010-01-07');
   * // 6
   * </code>
   *
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date_start
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date_end
   */
  public static function getDaysBetweenDates($date_start, $date_end): ?int {
    $date_start = self::createPhpDateTime($date_start);
    $date_end = self::createPhpDateTime($date_end);
    return ($date_start && $date_end) ? (int)$date_end->diff($date_start)->format('%a') : NULL;
  }

  /**
   * Return timestamp by date.
   *
   * Example:
   * <code>
   * $timestamp = DateHelper::getDateTimestamp('2010-01-01 UTC');
   * // 1262304000
   * </code>
   *
   * @param int|string|DrupalDateTime|PhpDateTimeInterface $date
   */
  public static function getDateTimestamp($date): ?int {
    if (is_int($date)) {
      return $date;
    }
    if (is_string($date)) {
      return strtotime($date);
    }
    if ($date instanceof DrupalDateTime || $date instanceof PhpDateTimeInterface) {
      return $date->getTimestamp();
    }
    return NULL;
  }

  /**
   * Return TRUE if $date_to_check is between $date_start and $date_end.
   *
   * Example:
   * <code>
   * $result = DateHelper::checkDateInDaterange('2010-06-01', '2010-01-01', '2010-12-31');
   * // TRUE
   * </code>
   *
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date_to_check
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date_start
   * @param string|integer|DrupalDateTime|PhpDateTimeInterface $date_end
   */
  public static function checkDateInDaterange($date_to_check, $date_start, $date_end): bool {
    $date_to_check = self::getDateTimestamp($date_to_check);
    $date_start = self::getDateTimestamp($date_start);
    $date_end = self::getDateTimestamp($date_end);

    if ($date_start === NULL) {
      $date_start = PHP_INT_MIN;
    }
    if ($date_end === NULL) {
      $date_end = PHP_INT_MAX;
    }

    return ($date_to_check >= $date_start && $date_to_check <= $date_end);
  }

}
