<?php

namespace Drupal\druhels;

use Drupal\Component\Utility\Timer;
use Drupal\Core\Controller\TitleResolverInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Symfony\Component\HttpFoundation\Request;

class DrupalHelper {

  /**
   * Return current page title.
   *
   * @return string|array String or render-array
   */
  public static function getCurrentPageTitle(Request $request = NULL) {
    if (!$request) {
      $request = \Drupal::request();
    }
    $route = $request->attributes->get(RouteObjectInterface::ROUTE_OBJECT);
    $title_resolver = \Drupal::service('title_resolver'); /** @var TitleResolverInterface $title_resolver */
    $page_title = $title_resolver->getTitle($request, $route);

    if (is_array($page_title) && isset($page_title['#markup'])) {
      $page_title = $page_title['#markup'];
    }

    return $page_title;
  }

  /**
   * Set page title.
   *
   * @param string $title Page title
   * @param boolean $unset_route_title_callback Set TRUE if must unset route title callback
   */
  public static function setCurrentPageTitle(string $title, bool $unset_route_title_callback = FALSE, RouteMatchInterface  $route_match = NULL): void {
    if (!$route_match) {
      $route_match = \Drupal::routeMatch();
    }
    $current_route = $route_match->getRouteObject();
    $current_route->setDefault('_title', $title);

    if ($unset_route_title_callback) {
      $current_route->setDefault('_title_callback', NULL);
    }
  }

  /**
   * Return current system path without trailing slashes.
   */
  public static function getCurrentSystemPath(): string {
    $current_system_path = \Drupal::service('path.current')->getPath();
    return ltrim($current_system_path, '/');
  }

  /**
   * Return current path alias without trailing slashes.
   */
  public static function getCurrentPathAlias(): string {
    $current_system_path = self::getCurrentSystemPath();
    $current_path_alias = \Drupal::service('path_alias.manager')->getAliasByPath('/' . $current_system_path);
    return ltrim($current_path_alias, '/');
  }

  /**
   * Return current route name.
   */
  public static function getCurrentRouteName(RouteMatchInterface $route_match = NULL): ?string {
    static $current_route_name;

    if ($current_route_name === NULL) {
      if ($route_match === NULL) {
        $route_match = \Drupal::routeMatch();
      }
      $current_route_name = $route_match->getRouteName();
    }

    return $current_route_name;
  }

  /**
   * Return TRUE if current route is admin route.
   */
  public static function isAdminRoute(bool $reset_cache = FALSE): bool {
    static $cache = NULL;

    if ($reset_cache) {
      $cache = NULL;
    }
    if ($cache === NULL) {
      $cache = \Drupal::service('router.admin_context')->isAdminRoute();
    }

    return $cache;
  }

  /**
   * Return TRUE if current page is front.
   */
  public static function isFrontPage(bool $reset_cache = FALSE): bool {
    static $cache = NULL;

    if ($reset_cache) {
      $cache = NULL;
    }
    if ($cache === NULL) {
      $cache = \Drupal::service('path.matcher')->isFrontPage();
    }

    return $cache;
  }

  /**
   * Start timer.
   *
   * Example:
   * <code>
   * DrupalHelper::timerStart();
   * sleep(1);
   * DrupalHelper::timerStop();
   * </code>
   */
  public static function timerStart(string $name = 'druhels'): void {
    Timer::start($name);
  }

  /**
   * Stop timer and show result.
   *
   * Example:
   * <code>
   * DrupalHelper::timerStart();
   * sleep(1);
   * DrupalHelper::timerStop();
   * </code>
   */
  public static function timerStop(string $name = 'druhels', bool $show_result = TRUE): void {
    Timer::stop($name);

    if ($show_result) {
      $duration = Timer::read($name);
      \Drupal::messenger()->addMessage($duration . ' ms / ' . number_format($duration / 1000, 5) . ' sec');
    }
  }

  /**
   * Return default langcode (language id).
   */
  public static function getDefaultLangcode(): string {
    static $default_language_id;

    if ($default_language_id === NULL) {
      $default_language_id = \Drupal::languageManager()->getDefaultLanguage()->getId();
    }

    return $default_language_id;
  }

  /**
   * Return current langcode (language id).
   */
  public static function getCurrentLangcode(): string {
    static $current_language_id;

    if ($current_language_id === NULL) {
      $current_language_id = \Drupal::languageManager()->getCurrentLanguage()->getId();
    }

    return $current_language_id;
  }

  /**
   * Return site mail address.
   */
  public static function getSiteMail(): string {
    return \Drupal::config('system.site')->get('mail');
  }

  /**
   * Send mail without need implements hook_mail().
   */
  public static function sendMail(string $to, string $subject, $message, string $key = 'druhels', string $langcode = NULL, array $params = []) {
    if (!$langcode) {
      $langcode = self::getDefaultLangcode();
    }

    $mail_service = \Drupal::service('plugin.manager.mail'); /* @var MailManagerInterface $mail_service */

    $params['subject'] = $subject;
    $params['body'] = $message;

    return $mail_service->mail('druhels', $key, $to, $langcode, $params);
  }

  /**
   * Return breakpoint media query.
   *
   * Example:
   * <code>
   * $wide_media_query = DrupalHelper::getBreakpointMediaQuery('seven', 'seven.wide');
   * // 'screen and (min-width: 40em)'
   * </code>
   */
  public static function getBreakpointMediaQuery(string $group, string $breakpoint_name): string {
    return \Drupal::service('breakpoint.manager')->getBreakpointsByGroup($group)[$breakpoint_name]->getMediaQuery();
  }

  /**
   * Render without change input data.
   *
   * Example:
   * <code>
   * $output = DrupalHelper::render(['#markup' => 'Hello']);
   * // 'Hello'
   * </code>
   *
   * @param array|mixed $build Render-array ot string
   */
  public static function render($build): string {
    return render($build);
  }

}
