<?php

namespace Drupal\druhels;

use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\TypedData\TranslatableInterface;
use Drupal\field\Entity\FieldStorageConfig;

class EntityHelper {

  /**
   * Return field label. Supported base and configurable fields.
   */
  public static function getFieldLabel(string $entity_type, string $bundle_name, string $field_name): string {
    $entity_field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $entity_field_manager */
    return $entity_field_manager->getFieldDefinitions($entity_type, $bundle_name)[$field_name]->getLabel();
  }

  /**
   * Return list field allowed values.
   *
   * Example:
   * <code>
   * $allowed_values = EntityHelper::getListFieldAllowedValues('node', 'page', 'field_list_integer');
   * // [
   * //   0 => 'Foo',
   * //   1 => 'Bar',
   * // ]
   * </code>
   */
  public static function getListFieldAllowedValues(string $entity_type, string $entity_bundle, string $field_name): array {
    $field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $field_manager */
    $field_definition = $field_manager->getFieldDefinitions($entity_type, $entity_bundle)[$field_name];
    $field_storage_definition = $field_definition->getFieldStorageDefinition();
    return options_allowed_values($field_storage_definition);
  }

  /**
   * Return value label for List field.
   */
  public static function getListFieldValueLabel(FieldItemListInterface $field_items, $delta = 0) {
    $allowed_values = options_allowed_values($field_items->getFieldDefinition()->getFieldStorageDefinition(), $field_items->getEntity());
    $value = $field_items->value;
    return ($value !== NULL && isset($allowed_values[$value])) ? $allowed_values[$value] : $value;
  }

  /**
   * Return field main values.
   *
   * Example:
   * <code>
   * $node->set('field_strings', ['Foo', 'Bar']);
   * $values = $node->get('field_strings')->getValues(); // [['value' => 'Foo'], ['value' => 'Bar']]
   * $main_values = EntityHelper::getFieldMainValues($node->get('field_strings')); // ['Foo', 'Bar']
   * </code>
   */
  public static function getFieldMainValues(FieldItemListInterface $items): array {
    if ($values = $items->getValue()) {
      $main_property_name = $items->first()->mainPropertyName() ?: key(current($values));
      return array_column($values, $main_property_name);
    }
    return [];
  }

  /**
   * Return field value.
   *
   * @return mixed
   */
  public static function getExistsFieldValue(FieldableEntityInterface $entity, string $field_name, string $value_name = 'value') {
    return $entity->hasField($field_name) ? $entity->get($field_name)->{$value_name} : NULL;
  }

  /**
   * Return TRUE if bundle has field.
   */
  public static function bundleHasField(string $entity_type, string $bundle, string $field_name): bool {
    $field_storage = FieldStorageConfig::loadByName($entity_type, $field_name);
    return $field_storage && in_array($bundle, $field_storage->getBundles());
  }

  /**
   * Return bundle field definitions by field type.
   *
   * @param string $entity_type_id
   * @param string $bundle
   * @param string $field_type
   *
   * @return FieldDefinitionInterface[] Key is field machine name
   */
  public static function getFieldDefinitionsByType(string $entity_type_id, string $bundle, string $field_type): array {
    $entity_field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManager $entity_field_manager */
    $field_definitions = $entity_field_manager->getFieldDefinitions($entity_type_id, $bundle);
    return array_filter($field_definitions, function ($field_definition) use ($field_type) {
      /** @var FieldDefinitionInterface $field_definition */
      return $field_definition->getType() == $field_type;
    });
  }

  /**
   * Return entity build array. Array does not contain fields, they generate in #pre_render.
   * If need array with builded fields use viewWithFields()
   *
   * @param EntityInterface $entity Entity object
   */
  public static function view(EntityInterface $entity, string $view_mode = 'full'): array {
    $entity_type_id = $entity->getEntityTypeId();
    return \Drupal::entityTypeManager()->getViewBuilder($entity_type_id)->view($entity, $view_mode);
  }

  /**
   * Return entity build array with fields.
   */
  public static function viewWithFields(EntityInterface $entity, string $view_mode = 'full'): array {
    $build = self::view($entity, $view_mode);

    foreach ($build['#pre_render'] as $callable) {
      $build = call_user_func($callable, $build);
    }

    unset($build['#pre_render']);

    return $build;
  }

  /**
   * Delete multiple entities.
   *
   * @param integer[] $entity_ids
   * @param string $entity_type Entity type id
   */
  public static function deleteMultiple(array $entity_ids, string $entity_type) {
    $entity_storage = \Drupal::entityTypeManager()->getStorage($entity_type);
    $entities = $entity_storage->loadMultiple($entity_ids);
    return $entity_storage->delete($entities);
  }

  /**
   * Return TRUE if entity field value changed.
   */
  public static function fieldHasAffectingChanges(EntityInterface $entity, string $field_name): bool {
    $original_entity = $entity->original; /** @var EntityInterface $original_entity */
    $field_items = $entity->get($field_name);
    $original_field_items = $original_entity->get($field_name);

    return $field_items->hasAffectingChanges($original_field_items, $field_items->getLangcode());
  }

  /**
   * Return TRUE if list field has certain value.
   */
  public static function listFieldHasValue(FieldItemListInterface $items, $value): bool {
    return in_array($value, self::getFieldMainValues($items));
  }

  /**
   * Return content entity types.
   *
   * @return ContentEntityTypeInterface[] Key is entity type id
   */
  public static function getContentEntityTypes(): array {
    return array_filter(\Drupal::entityTypeManager()->getDefinitions(), function (EntityTypeInterface $entity_type) {
      return $entity_type instanceof ContentEntityTypeInterface;
    });
  }

  /**
   * Return TRUE if current page is entity canonical page.
   */
  public static function isEntityPage($entity_type_id = NULL): bool {
    if (preg_match('/^entity\.(.+)\.canonical$/', DrupalHelper::getCurrentRouteName(), $matches)) {
      if ($entity_type_id) {
        return $matches[1] == $entity_type_id;
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Return current entity from route.
   */
  public static function getCurrentEntity(RouteMatchInterface $route_match = NULL): ?EntityInterface {
    if (!$route_match) {
      $route_match = \Drupal::routeMatch();
    }

    if (preg_match('/^entity\.(.+)\.canonical$/', DrupalHelper::getCurrentRouteName($route_match), $matches)) {
      $entity_type_id = $matches[1];
      $entity = $route_match->getParameter($entity_type_id); /** @var EntityInterface $entity */

      if ($entity && is_numeric($entity)) {
        $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity);
      }

      return $entity;
    }

    return NULL;
  }

  /**
   * Return translated entity.
   */
  public static function getTranslatedEntity(TranslatableInterface $entity, $langcode = NULL): EntityInterface {
    if (!$langcode) {
      $langcode = DrupalHelper::getCurrentLangcode();
    }

    return $entity->hasTranslation($langcode) ? $entity->getTranslation($langcode) : $entity;
  }

  /**
   * Set third party settings.
   */
  public static function setThirdPartySettings(ThirdPartySettingsInterface $entity, string $module_name, array $settings): void {
    foreach ($settings as $key => $value) {
      $entity->setThirdPartySetting($module_name, $key, $value);
    }
  }

}
